﻿; *---------------------------------*
; *   OpenGL Base Example           *
; *   xorc1zt                       *
; *   2011                          *
; *---------------------------------*

EnableExplicit
IncludeFile "glbase.pb"

Global quitmainloop.b = #False
Global viewmatrix.MATRIX
Global projmatrix.MATRIX
Global program_id.i
Global texture_id.i

;- Declarations
Declare tickWindow()
Declare onResize( height.i, width.i )
Declare onRender()
Declare onInit()

;- Start
pbglOpenWindow( 100, 100, 800, 600, "Hello OpenGL" )
onInit()

;- Main loop
Repeat
    onRender()
    FlipBuffers()
    tickWindow()
Until quitmainloop

;- Procedures
Procedure tickWindow()
    Select WindowEvent()
        Case #PB_Event_CloseWindow
            quitmainloop = #True
        Case #PB_Event_SizeWindow
            onResize( WindowWidth(0),  WindowHeight(0) )
    EndSelect
EndProcedure

Procedure onResize( width.i,  height.i )
    glViewport( 0, 0, width, height )
    projmatrix\perspective( 70, 1, 100, width/height )
EndProcedure

Procedure onRender()
    Define vertex_location.i
    Define color_location.i
    Define texturecoord_location.i

    Static angle.f
    
    glClearScreen( 0.2, 0.2, 0.2 )

    angle - 1.0
    
    vertex_location = glGetAttribLocation( program_id, "in_Vertex" )
    glVertexAttribPointer( vertex_location, 3, #GL_FLOAT, #GL_FALSE, 0, ?cubevertex )
    glEnableVertexAttribArray( vertex_location )
    
    texturecoord_location = glGetAttribLocation( program_id, "in_TexCoord0" )
    glVertexAttribPointer( texturecoord_location, 2, #GL_FLOAT, #GL_FALSE, 0, ?cubetexture  )
    glEnableVertexAttribArray( texturecoord_location )
    
    glUseProgram( program_id )
    glActiveTexture(#GL_TEXTURE0)
    glBindTexture(#GL_TEXTURE_2D, texture_id)
    glUniform1i(glGetUniformLocation(program_id, "texture"), 0);
    
    glUniformMatrix4fv( glGetUniformLocation( program_id, "projection" ), 1, #GL_FALSE, projmatrix\getAddress() )
    
    viewmatrix\identity()
    viewmatrix\lookAt( 0, 3, 15, 0, 0, 0, 0, 1, 0)
    
    ;cube 1 at {0,0,0}
    viewmatrix\push()
    viewmatrix\rotateZ(angle)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\pop()
    
    ;cube 2 at {-3,0,0}
    viewmatrix\push()
    viewmatrix\translate(-3,0,0)
    viewmatrix\rotateY(angle)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\pop()
    
    ;cube 3 at {3,0,0}
    viewmatrix\push()
    viewmatrix\translate(3,0,0)
    viewmatrix\rotateX(angle)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\pop()
    
    viewmatrix\push()
    viewmatrix\translate(0,3,-5)
    viewmatrix\rotateX(-angle)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\translate(3,0,0)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\translate(-6,0,0)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\pop()
    
    viewmatrix\push()
    viewmatrix\translate(0,-3,5)
    viewmatrix\rotateY(-angle)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\translate(3,0,0)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\translate(-6,0,0)
    glUniformMatrix4fv( glGetUniformLocation( program_id, "modelview" ), 1, #GL_FALSE, viewmatrix\getAddress() ) 
    glDrawArrays( #GL_TRIANGLES, 0, 36 )
    viewmatrix\pop()
    
    glUseProgram( 0 )
    
    glDisableVertexAttribArray( vertex_location )
    glDisableVertexAttribArray( texturecoord_location )
    
EndProcedure

Procedure onInit()
    Define vertex_shader.s
    Define fragment_shader.s
    ;glDisable(#GL_TEXTURE_RECTANGLE)
    ;initialize our matrix
    viewmatrix = createMatrix("viewmatrix")
    projmatrix = createMatrix("projmatrix")
    
    ;vertex shader source code
    vertex_shader = "#version 330"+#CRLF$
    vertex_shader + "in vec3 in_Vertex;"+#CRLF$
    vertex_shader + "in vec2 in_TexCoord0;"+#CRLF$
    vertex_shader + "out vec2 texCoord0;"+#CRLF$
    vertex_shader + "uniform mat4 projection;"+#CRLF$
    vertex_shader + "uniform mat4 modelview;"+#CRLF$
    vertex_shader + "void main()"+#CRLF$
    vertex_shader + "{"+#CRLF$
    vertex_shader + "gl_Position = projection * modelview * vec4(in_Vertex, 1.0);"+#CRLF$
    vertex_shader + "texCoord0 = in_TexCoord0;"+#CRLF$
    vertex_shader + "}"
    
    ;fragment shader source code
    fragment_shader = "#version 330"+#CRLF$
    fragment_shader + "in vec2 texCoord0;"+#CRLF$
    fragment_shader + "out vec3 out_Color;"+#CRLF$
    fragment_shader + "uniform sampler2D texture;"+#CRLF$
    fragment_shader + "void main()"+#CRLF$
    fragment_shader + "{"+#CRLF$
    fragment_shader + "out_Color = texture(texture, texCoord0).rgb;"+#CRLF$
    fragment_shader + "}"
    
    ;create a program with our shaders
    program_id = pbglCreateProgramFast(vertex_shader, fragment_shader)
    
    ;load our texture  
    texture_id = pbglLoadTexture("res\uvtemplate.tga")  
EndProcedure

DataSection
    
    cubetexture:
    Data.f 0.000059, 1.0-0.000000; 1
    Data.f 0.335973, 1.0-0.335903
    Data.f 0.336098, 1.0-0.000071
    Data.f 0.000059, 1.0-0.000004 
    Data.f 0.000103, 1.0-0.336048
    Data.f 0.335973, 1.0-0.335903
    Data.f 0.335973, 1.0-0.335903; 2
    Data.f 0.668104, 1.0-0.000013
    Data.f 0.336098, 1.0-0.000071
    Data.f 0.668104, 1.0-0.000013
    Data.f 0.335973, 1.0-0.335903
    Data.f 0.667979, 1.0-0.335851
    Data.f 1.000023, 1.0-0.000013; 3
    Data.f 0.668104, 1.0-0.000013
    Data.f 0.667979, 1.0-0.335851
    Data.f 1.000023, 1.0-0.000013
    Data.f 0.667979, 1.0-0.335851
    Data.f 0.999958, 1.0-0.336064
    Data.f 0.000103, 1.0-0.336048; 4
    Data.f 0.000004, 1.0-0.671870
    Data.f 0.336024, 1.0-0.671877
    Data.f 0.000103, 1.0-0.336048
    Data.f 0.336024, 1.0-0.671877
    Data.f 0.335973, 1.0-0.335903
    Data.f 0.667979, 1.0-0.335851; 5
    Data.f 0.335973, 1.0-0.335903
    Data.f 0.336024, 1.0-0.671877
    Data.f 0.667979, 1.0-0.335851
    Data.f 0.336024, 1.0-0.671877
    Data.f 0.667969, 1.0-0.671889  
    Data.f 0.667969, 1.0-0.671889; 6
    Data.f 1.000004, 1.0-0.671847 
    Data.f 0.667979, 1.0-0.335851
    Data.f 1.000004, 1.0-0.671847
    Data.f 0.999958, 1.0-0.336064
    Data.f 0.667979, 1.0-0.335851
    
    cubevertex:
    Data.f  1.0, 1.0,-1.0; north
    Data.f -1.0,-1.0,-1.0
    Data.f -1.0, 1.0,-1.0
    Data.f  1.0, 1.0,-1.0
    Data.f  1.0,-1.0,-1.0
    Data.f -1.0,-1.0,-1.0  
    Data.f -1.0,-1.0,-1.0; west
    Data.f -1.0, 1.0, 1.0
    Data.f -1.0, 1.0,-1.0
    Data.f -1.0, 1.0, 1.0  
    Data.f -1.0,-1.0,-1.0
    Data.f -1.0,-1.0, 1.0 
    Data.f  1.0,-1.0,-1.0; east 
    Data.f  1.0,-1.0, 1.0
    Data.f  1.0, 1.0, 1.0
    Data.f  1.0,-1.0,-1.0
    Data.f  1.0, 1.0, 1.0
    Data.f  1.0, 1.0,-1.0
    Data.f -1.0, 1.0, 1.0; south
    Data.f -1.0,-1.0, 1.0
    Data.f  1.0,-1.0, 1.0 
    Data.f -1.0, 1.0, 1.0
    Data.f  1.0,-1.0, 1.0
    Data.f  1.0, 1.0, 1.0
    Data.f  1.0, 1.0, 1.0; top
    Data.f  1.0, 1.0,-1.0
    Data.f -1.0, 1.0,-1.0    
    Data.f  1.0, 1.0, 1.0
    Data.f -1.0, 1.0,-1.0
    Data.f -1.0, 1.0, 1.0
    Data.f  1.0,-1.0,-1.0; bot
    Data.f  1.0,-1.0, 1.0
    Data.f -1.0,-1.0,-1.0
    Data.f  1.0,-1.0, 1.0
    Data.f -1.0,-1.0, 1.0
    Data.f -1.0,-1.0,-1.0 
EndDataSection