﻿; *---------------------------------*
; *   OpenGL Base Example           *
; *   xorc1zt                       *
; *   2012                          *
; *---------------------------------*

EnableExplicit
IncludeFile "glbase.pb"

Global quitmainloop.b = #False
Global projmatrix.MATRIX
Global texture_id.i
Global orthoshader_id.i

;- Declarations
Declare tickWindow()
Declare onResize( height.i, width.i )
Declare onRender()
Declare onInit()
Declare DrawSprite( texture_id.i, x, y, w, h )

;- Start
pbglSetting( #PBGL_GLVERSION_MAJOR, 3 )
pbglSetting( #PBGL_GLVERSION_MINOR, 3 )
pbglSetting( #PBGL_GLPROFILE, #PBGL_GLPROFILE_COMPATIBILITY )
pbglSetting( #PBGL_CONTEXT_CREATION, #PBGL_WINAPI )
pbglOpenWindow( 100, 100, 800, 600, "Hello OpenGL" )
onInit()

;- Main loop
Repeat
    onRender()
    pbglSwapBuffers()
    tickWindow()
Until quitmainloop

;- Procedures
Procedure tickWindow()
    Select WindowEvent()
        Case #PB_Event_CloseWindow
            quitmainloop = #True
        Case #PB_Event_SizeWindow
            onResize( WindowWidth( 0 ),  WindowHeight( 0 ) )
    EndSelect
EndProcedure

Procedure onResize( width.i,  height.i )
    glViewport( 0, 0, width, height )
    ;projmatrix\perspective( 70.0, 1.0, 100.0, width/height )
    projmatrix\orthogonal( 0, width, 0, height, -1, 1 )
    
    glUseProgram( orthoshader_id )
    glUniformMatrix4fv( glGetUniformLocation( orthoshader_id, "projection" ), 1, #GL_FALSE, projmatrix\getAddress() ) 
    glUseProgram( 0 )
EndProcedure

Procedure onRender()
    Define counter.i
    Static angle.f
    Define delta.f = pbglGetTimeDelta()
    
    glClearScreen( 0, 0, 0.3 )
 
;     For counter = 0 To 2
;         DrawSprite( texture_id, Random( 400 ) , Random( 300 ) , Random( 399 )+1, Random( 299 )+1 )
;     Next counter
    DrawSprite( texture_id, 0 , 0, 600, 600 )
    DrawSprite( texture_id, 100 , 500, 100, 100 )
    DrawSprite( texture_id, 200 , 500, 100, 100 )
    DrawSprite( texture_id, 300 , 500, 100, 100 )
    DrawSprite( texture_id, 400 , 500, 100, 100 )
    DrawSprite( texture_id, 100 , 100, 100, 100 )
    DrawSprite( texture_id, 200 , 100, -100, 100 )
    DrawSprite( texture_id, 300 , 100, 100, -100 )
    DrawSprite( texture_id, 400 , 100, -100, -100 )
    DrawSprite( texture_id, 500 , 100, 100, 100 )
    DrawSprite( texture_id, 500 , 200, 100, -100 )
    
    angle + ( 1.5 * delta )
    
    DrawSprite( texture_id, 400+Sin(angle)*50, 400+Cos(angle)*50, Cos(angle)*100, Sin(angle)*100 )
    DrawSprite( texture_id, 400+Cos(angle)*50, 400+Sin(angle)*50, Sin(angle)*100, Cos(angle)*100 )
    DrawSprite( texture_id, 600, 300, Sin(angle)*200, Sin(angle)*200 )
EndProcedure

Procedure onInit()
    Define vertex_shader.s
    Define fragment_shader.s

    ;initialize our matrix
    projmatrix = createMatrix( "projmatrix" )
    
    ;create a program with our shaders
    orthoshader_id  = pbglCreateProgramFast( "shader\ortho.vert", "shader\ortho.frag", #True )
    
    ;load our texture  
    texture_id = pbglLoadTexture( "res\sword.png", #GL_TEXTURE_2D, #GL_NEAREST, #GL_NEAREST )  

    ;enable alpha channel
    glEnable( #GL_BLEND )
    glBlendFunc( #GL_SRC_ALPHA, #GL_ONE_MINUS_SRC_ALPHA ) 
EndProcedure

Procedure DrawSprite( texture_id.i, x, y, w, h )
    Dim quad.l(7)
    Define in_Vertex_location.i
    Define in_TexCoord0_location.i
    Define texture_location.i
    Define TextureID_location.i
    
    in_Vertex_location      = glGetAttribLocation( orthoshader_id, "in_Vertex3D" )
    in_TexCoord0_location   = glGetAttribLocation( orthoshader_id, "in_TexCoord0" )
    TextureID_location      = glGetUniformLocation( orthoshader_id, "TextureID" )
    
    glUseProgram( orthoshader_id )
    glActiveTexture( #GL_TEXTURE0 )
    glBindTexture( #GL_TEXTURE_2D, texture_id )
    glUniform1i( TextureID_location , 0 );
    glVertexAttribIPointer( in_Vertex_location, 2, #GL_INT, 0, quad() )
    glVertexAttribPointer( in_TexCoord0_location, 2, #GL_FLOAT, #GL_FALSE, 0, ?texture_coord  )
    glEnableVertexAttribArray( in_Vertex_location )
    glEnableVertexAttribArray( in_TexCoord0_location )
    
    quad(0) = x
    quad(1) = y
    quad(2) = x+w
    quad(3) = y
    quad(4) = x+w
    quad(5) = y+h
    quad(6) = x
    quad(7) = y+h
    
    glDrawArrays( #GL_QUADS, 0,  4 )
    glUseProgram( 0 )
    glDisableVertexAttribArray( in_Vertex_location )
    glDisableVertexAttribArray( in_TexCoord0_location )
EndProcedure

DataSection
    cubevertex:
    Data.l 100, 100
    Data.l 200, 100
    Data.l 200, 200
    Data.l 100, 200
    
    texture_coord:
    Data.f 0.0, 0.0
    Data.f 1.0, 0.0
    Data.f 1.0, 1.0
    Data.f 0.0, 1.0
EndDataSection